@@ -1,4 +1,4 @@
-#!/usr/bin/python3.12
+#!/usr/bin/python3.13
 '''
 Provides IPython console widget.
 
@@ -177,7 +177,8 @@ class IterableIPShell:
 
     self.prompt = self.generatePrompt(self.iter_more)
 
-    self.IP.hooks.pre_prompt_hook()
+    if IPython.version_info < (9,):
+        self.IP.hooks.pre_prompt_hook()
     if self.iter_more:
         try:
             self.prompt = self.generatePrompt(True)
@@ -189,7 +190,7 @@ class IterableIPShell:
     try:
       line = self.IP.raw_input(self.prompt)
     except KeyboardInterrupt:
-      self.IP.write('\nKeyboardInterrupt\n')
+      self.write('\nKeyboardInterrupt\n')
       if self.no_input_splitter:
         self.lines = []
       else:
@@ -335,7 +336,12 @@ class IterableIPShell:
         return str1
       if possibilities[1]:
         common_prefix = reduce(_commonPrefix, possibilities[1]) or split_line[-1]
-        completed = line[:-len(split_line[-1])]+common_prefix
+        autocomplete_start_index = line.rfind(possibilities[0])
+        completed = line[0:autocomplete_start_index] + common_prefix
+        # suggestions for current line consist of text not used for completion + completion matches
+        unmatched_text = line[-len(split_line[-1]):autocomplete_start_index]
+        suggestions = [unmatched_text + p for p in possibilities[1]]
+        possibilities = (possibilities[0], suggestions)
       else:
         completed = line
     else:
@@ -475,6 +481,8 @@ class ConsoleView(gtk.TextView):
       ansi_tags = self.color_pat.findall(text)
       for tag in ansi_tags:
         i = segments.index(tag)
+        if tag not in self.ANSI_COLORS:
+          tag = '0;30'
         self.text_buffer.insert_with_tags_by_name(self.text_buffer.get_end_iter(),
                                              segments[i+1], tag)
         segments.pop(i)
